 /*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2003 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 2.02 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/2_02.txt.                                 |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author:     Alexandre Kalendarev  akalend@mail.ru                          |
  +----------------------------------------------------------------------+

  $Id: header,v 1.10.8.1 2003/07/14 15:59:18 sniper Exp $ 
*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_xmlsec.h"


/* If you declare any globals in php_xmlsec.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(xmlsec)
*/

/* True global resources - no need for thread safety here */
static int le_xmlseckey;
static int le_xmlsectmpl;
static int le_xmlsecdoc;


typedef struct _php_xmlsec_key {
	int type;
	char name[32];	
	char key[2048];	
        xmlSecKeyPtr p_key;
} php_xmlsec_key;

typedef struct _php_xmlsec_tpl {
	xmlDoc*  doc ;	
	php_xmlsec_key* p_key;
} php_xmlsec_tpl;

typedef struct _php_xmlsec_doc {
	xmlDoc*  doc ;	
	php_xmlsec_key* p_key;
	php_xmlsec_tpl* p_tpl;
} php_xmlsec_doc;


//static void xmlsec_error(void *ctx, const char *msg,  );

zend_class_entry *xmlsec_doc_class_entry;
zend_class_entry *xmlsec_key_class_entry;
zend_class_entry *xmlsec_template_class_entry;

/* {{{ xmlsec_functions[]
 *
 * Every user visible function must have an entry in xmlsec_functions[].
 */
function_entry xmlsec_functions[] = {
	PHP_FE( xmlsec_key_ini,				NULL)
	PHP_FE( xmlsec_key_set_name,		NULL)
	PHP_FE( xmlsec_key_get_name,		NULL)
	PHP_FE( xmlsec_key_load,			NULL)
	PHP_FE(xmlsec_template_set_key,	NULL)	
	PHP_FE(xmlsec_template_ini, 		NULL)
	PHP_FE(xmlsec_template_getdoc,	NULL)		 
	PHP_FE(xmlsec_template_enc_create,	NULL)		 	
	PHP_FE(xmlsec_template_dsig_create,	NULL)		 		
	PHP_FE( xmlsec_doc_ini,			NULL)	
	PHP_FE( xmlsec_doc_set_template,			NULL)		
	PHP_FE( xmlsec_doc_encrypt,			NULL)		
	{NULL, NULL, NULL}	/* Must be the last line in xmlsec_functions[] */
};

static function_entry php_xmlsec_doc_class_functions[] = {
	PHP_FALIAS( testDoc,				xmlsec_doc_ini,				NULL)
	{NULL, NULL, NULL}
};
static function_entry php_xmlsec_key_class_functions[] = {
	PHP_FALIAS( setName,				xmlsec_key_set_name,			NULL)
	PHP_FALIAS( getName,				xmlsec_key_get_name,			NULL)
	PHP_FALIAS( key,				xmlsec_key_get_name,			NULL)	
	{NULL, NULL, NULL}
};
static function_entry php_xmlsec_tmpl_class_functions[] = {
	PHP_FALIAS( TmplIni,				xmlsec_template_ini,			NULL)
	PHP_FALIAS( getDoc,				xmlsec_template_getdoc,			NULL)	
	{NULL, NULL, NULL}
};


/* }}} */

/* {{{ xmlsec_module_entry
 */
zend_module_entry xmlsec_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"xmlsec",
	xmlsec_functions,
	PHP_MINIT(xmlsec),
	PHP_MSHUTDOWN(xmlsec),
	PHP_RINIT(xmlsec),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(xmlsec),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(xmlsec),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_XMLSEC
ZEND_GET_MODULE(xmlsec)
#endif

/* destructor of  xmlsec_key*/
static void php_free_xmlsec_key(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	php_xmlsec_key *keyPtr ;
	
	keyPtr = (php_xmlsec_key*) rsrc->ptr;
//	fprintf(  stderr, "  destructor free  %x  \n",  keyPtr );	
	if ( keyPtr) { 
	  efree( keyPtr);
	}
}

/* destructor of  xmlsec_template*/
static void php_free_xmlsec_tpl(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	php_xmlsec_tpl *tplPtr =NULL;
	xmlDoc* pDoc  = NULL;
	
	tplPtr = (php_xmlsec_tpl*) rsrc->ptr;
//	fprintf(  stderr, "  destructor free  %x  \n",  keyPtr );	
	if ( tplPtr) { 
		if (  tplPtr->doc != NULL )
			pDoc  = ( xmlDoc*) tplPtr->doc ;			
		if ( pDoc != NULL ){
			xmlFreeDoc(pDoc);
//			fprintf(stdout, "destructor DOM OK");
		}	
	  	efree( tplPtr);
	}

}


/* destructor of  xmlsec_doc*/
static void php_free_xmlsec_doc(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{

	php_xmlsec_doc* docPtr =NULL;
	xmlDoc* pDoc  = NULL;
	
	docPtr  = (php_xmlsec_doc*) rsrc->ptr;
//	fprintf(  stderr, "  destructor free  %x  \n",  keyPtr );	
	if ( docPtr) { 
		if (  docPtr->doc != NULL )
			pDoc  = ( xmlDoc*) docPtr->doc ;			
		if ( pDoc != NULL ){
			xmlFreeDoc(pDoc);
		}	
	  	efree( docPtr);
			
	}

/*
xmlsecCtxRsc *xmlsecCtxt = ( xmlsecCtxRsc*)  rsrc->ptr;

if (  xmlsecCtxt ->encCtx !=NULL  )
    xmlSecEncCtxDestroy( xmlsecCtxt->encCtx );

if (  xmlsecCtxt ->dsigCtx !=NULL  )
    xmlSecDSigCtxDestroy( xmlsecCtxt->dsigCtx );
*/
}


/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("xmlsec.global_value",      "42", PHP_INI_ALL, OnUpdateInt, global_value, zend_xmlsec_globals, xmlsec_globals)
    STD_PHP_INI_ENTRY("xmlsec.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_xmlsec_globals, xmlsec_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_xmlsec_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_xmlsec_init_globals(zend_xmlsec_globals *xmlsec_globals)
{
	xmlsec_globals->global_value = 0;
	xmlsec_globals->global_string = NULL;
}
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(xmlsec)
{
	/* If you have INI entries, uncomment these lines 
	ZEND_INIT_MODULE_GLOBALS(xmlsec, php_xmlsec_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/
	zend_class_entry ce;
	
    	le_xmlseckey = zend_register_list_destructors_ex(php_free_xmlsec_key, NULL, "xmlseckey", module_number);
    	le_xmlsecdoc = zend_register_list_destructors_ex(php_free_xmlsec_doc, NULL, "xmlsecdoc", module_number);
   	le_xmlsectmpl = zend_register_list_destructors_ex(php_free_xmlsec_tpl, NULL, "xmlsectmpl", module_number);

		
	INIT_OVERLOADED_CLASS_ENTRY(ce, "xmlsec_key", php_xmlsec_key_class_functions, NULL, NULL, NULL);
	xmlsec_key_class_entry = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	
	INIT_OVERLOADED_CLASS_ENTRY(ce, "xmlsec_doc", php_xmlsec_doc_class_functions, NULL, NULL, NULL);
	xmlsec_doc_class_entry= zend_register_internal_class_ex(&ce, xmlsec_key_class_entry , NULL TSRMLS_CC);

	INIT_OVERLOADED_CLASS_ENTRY(ce, "xmlsectmpl", php_xmlsec_tmpl_class_functions, NULL, NULL, NULL);
	xmlsec_template_class_entry= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);

			
/*  default is DES algorithm  
#define xmlSecKeyDataAesId  
#define xmlSecKeyDataDesId  
#define xmlSecKeyDataDsaId 
#define xmlSecKeyDataHmacId
#define xmlSecKeyDataRsaId    
#define     xmlSecKeyDataX509Id
#define     xmlSecKeyDataRawX509CertId
#define     xmlSecX509StoreId    

	*/
	REGISTER_LONG_CONSTANT("XMLSEC_DES",	 XMLSEC_DES,  CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("XMLSEC_AES",	 XMLSEC_AES,  CONST_CS | CONST_PERSISTENT);
	
//	REGISTER_LONG_CONSTANT("XMLSEC_DCA",	 XMLSEC_DCA,  CONST_CS | CONST_PERSISTENT);
//	REGISTER_LONG_CONSTANT("XMLSEC_RSA",	 XMLSEC_RSA,  CONST_CS | CONST_PERSISTENT);
	
/* xmlSecTransformAes128CbcId:  The AES 128,192,256 CBC cipher transform klass. */	
	REGISTER_LONG_CONSTANT("XMLSEC_AES_128",  XMLSEC_AES128Cbc ,  CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("XMLSEC_AES_192",  XMLSEC_AES192Cbc ,  CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("XMLSEC_AES_256",  XMLSEC_AES256Cbc ,  CONST_CS | CONST_PERSISTENT);	
 
 
 /*   #define     xmlSecTransformDes3CbcId:  Triple DES CBC encryption transform klass.  */
	REGISTER_LONG_CONSTANT("XMLSEC_3DES",  XMLSEC_3DESCbc ,  CONST_CS | CONST_PERSISTENT);

/*  #define     xmlSecTransformSha1Id  */	
	REGISTER_LONG_CONSTANT("XMLSEC_SHA1",  XMLSEC_SHA1 ,  CONST_CS | CONST_PERSISTENT);

/*  #define xmlSecKeyDataDsaId    */
	REGISTER_LONG_CONSTANT("XMLSEC_DSA",  XMLSEC_DSA ,  CONST_CS | CONST_PERSISTENT);

/*	#define     xmlSecTransformDsaSha1Id	*/
	REGISTER_LONG_CONSTANT("XMLSEC_DSA_SHA1",  XMLSEC_DSA_SHA1 ,  CONST_CS | CONST_PERSISTENT);

/*	#define     xmlSecTransformRsaSha1Id	*/
	REGISTER_LONG_CONSTANT("XMLSEC_RSA_SHA1",  XMLSEC_RSA_SHA1 ,  CONST_CS | CONST_PERSISTENT);
	
/*	xmlSecTransformRsaOaepId	*/
REGISTER_LONG_CONSTANT("XMLSEC_TR_RSA_OAEP",  XMLSEC_TR_RSA_OAEP ,  CONST_CS | CONST_PERSISTENT);


/*	encrypt	*/
REGISTER_LONG_CONSTANT("XMLSEC_ENC",  XMLSEC_ENC ,  CONST_CS | CONST_PERSISTENT);

/*	#define     xmlSecTransformRsaOaepId   */
REGISTER_LONG_CONSTANT("XMLSEC_EN_RSA_OAEP_MGF1P",  XMLSEC_EN_RSA_OAEP_MGF1P,  CONST_CS | CONST_PERSISTENT);

		
/*

#define     XMLSEC_ENC						0x8000	
#define     XMLSEC_EN_RSA_OAEP_MGF1P	0x90B0	
#define     XMLSEC_ENC_DH					0x8040	

#define  XMLSEC_ENC_KW_3DES			0x8201	
#define  XMLSEC_ENC_KW_AES    			0x8101
#define  XMLSEC_ENC_KW_AES    			0x8102 
#define  XMLSEC_ENC_KW_AES    			0x8103


#define  XMLSEC_SHA1		    			0x0400
#define  XMLSEC_HMAC_SHA1		    	0x0480

#define  XMLSEC_DSA		    			0x0800
#define  XMLSEC_RSA		    			0x1000

#define  XMLSEC_DSA_SHA1	    			0x0B00
#define  XMLSEC_RSA_SHA1	    			0x1400

#define  XMLSEC_TR_XPATH				0x4000
#define  XMLSEC_TR_XSLT				0x4001
#define  XMLSEC_TR_ENV_SIG			0x4002

#define  XMLSEC_TR_C14n				0x4010
#define  XMLSEC_TR_C14n_COMMENTS	0x4011
*/		
	
/* not implementation	
#define xmlSecKeyDataAesId  
#define xmlSecKeyDataDesId  
#define xmlSecKeyDataDsaId 
#define xmlSecKeyDataHmacId
#define xmlSecKeyDataRsaId    
#define     xmlSecKeyDataX509Id
#define     xmlSecKeyDataRawX509CertId
#define     xmlSecX509StoreId    
                                         

#define     xmlSecTransformKWAes128Id
#define     xmlSecTransformKWAes192Id
#define     xmlSecTransformKWAes256Id
#define     xmlSecTransformDes3CbcId
#define     xmlSecTransformKWDes3Id
#define     xmlSecTransformDsaSha1Id
#define     xmlSecTransformHmacSha1Id
#define     xmlSecTransformHmacRipemd160Id
#define     xmlSecTransformHmacMd5Id
#define     xmlSecTransformRipemd160Id
#define     xmlSecTransformRsaSha1Id
#define     xmlSecTransformRsaPkcs1Id
#define     xmlSecTransformRsaOaepId
#define     xmlSecTransformSha1Id
*/


	
        xmlInitParser();
	
    /* Init libxml and libxslt libraries */
    LIBXML_TEST_VERSION
    xmlLoadExtDtdDefaultValue = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
    xmlSubstituteEntitiesDefault(1);
#ifndef XMLSEC_NO_XSLT
    xmlIndentTreeOutput = 1; 
#endif /* XMLSEC_NO_XSLT */

    if(xmlSecInit() < 0) {    	
	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot fetch DOM objec"); 
	return ;
    }
    
    /* Check loaded library version */
    if(xmlSecCheckVersion() != 1) {
	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error: loaded xmlsec library version is not compatible.\n"); 
	return ;
    }

    /* Load default crypto engine if we are supporting dynamic
     * loading for xmlsec-crypto libraries. Use the crypto library
     * name ("openssl", "nss", etc.) to load corresponding 
     * xmlsec-crypto library.
     */
#ifdef XMLSEC_CRYPTO_DYNAMIC_LOADING
    if(xmlSecCryptoDLLoadLibrary(BAD_CAST XMLSEC_CRYPTO) < 0) {
	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error: unable to load default xmlsec-crypto library. Check the LD_LIBRARY_PATH envornment variable\n  "); 
	return ;
	
    }
#endif /* XMLSEC_CRYPTO_DYNAMIC_LOADING */


    /* Init crypto library */
    if(xmlSecCryptoAppInit(NULL) < 0) {
	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error: crypto initialization failed.\n "); 
	return ;
    }

    /* Init xmlsec-crypto library */
    if(xmlSecCryptoInit() < 0) {
php_error_docref(NULL TSRMLS_CC, E_WARNING, "Error: xmlsec-crypto initialization failed\n "); 
	return ;
    }

    
	return SUCCESS;

}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(xmlsec)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(xmlsec)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(xmlsec)
{
    /* Shutdown xmlsec-crypto library */
    xmlSecCryptoShutdown();
    
    /* Shutdown crypto library */
    xmlSecCryptoAppShutdown();
    
    /* Shutdown xmlsec library */
    xmlSecShutdown();

    /* Shutdown libxslt/libxml */
#ifndef XMLSEC_NO_XSLT
    xsltCleanupGlobals();            
#endif /* XMLSEC_NO_XSLT */
    xmlCleanupParser();

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(xmlsec)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "xmlsec support", "enabled");
	php_info_print_table_end();


	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

    /* {{{ proto string xmlsec_template_ini( int arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_template_ini )
{
	char *arg = NULL;
	int len, k,arg_len;
	int rsc_id;
	char string[256];	
	php_xmlsec_tpl *tpl_rsc, *tpl_rsc2;
	zval **result ,  *tpl_result;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &arg, &arg_len ) == FAILURE) {
		return;
	}

	tpl_rsc = (php_xmlsec_tpl* )  emalloc(sizeof( php_xmlsec_tpl) );
	if ( ZEND_NUM_ARGS() == 1 )
	{		
		xmlDoc*pDoc =   xmlParseMemory( arg, arg_len);
		if ( !pDoc){
		 php_error(E_ERROR, "XmlDom:  c'not parse xml document" );		
		}
		//fprintf(  stderr, "the doc addr %x \n",  pDoc);
		tpl_rsc->doc =  pDoc;
	}
//	fprintf(  stderr, "is it template rsc Addr  %x  \n",  tpl_rsc );
	
	  ZEND_REGISTER_RESOURCE(return_value , tpl_rsc ,  le_xmlsectmpl  );
}
/* }}} */

/* {{{ proto string xmlsec_key_set_key( rsrc template , rsrc key)
       this is test function */
PHP_FUNCTION( xmlsec_template_set_key )
{
	int *arg = NULL;
	int len, k;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc, *tpl_rsc2;
	zval **result ,  *tpl_result , **result_2 , *key_result;
	xmlDoc* pDoc ;
	char  *docStr;
	php_xmlsec_key*  p_key;
	
	if ( ZEND_NUM_ARGS() != 2 ) WRONG_PARAM_COUNT;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr", &result  , &result_2  ) == FAILURE) {
		php_error(E_ERROR, "The arguments is wrong \n" );
		return;
	}	 

	zend_get_parameters_ex(2, &result  , &result_2);	
	
	ZEND_FETCH_RESOURCE( tpl_result , php_xmlsec_tpl*, result , -1, "xmlsec template", le_xmlsectmpl  );	 
	tpl_rsc  = (php_xmlsec_tpl* ) tpl_result ;
	pDoc = (xmlDoc* ) tpl_rsc ->doc;
	
		
	ZEND_FETCH_RESOURCE( key_result,  php_xmlsec_key*, result_2 , -1, "xmlsec results", le_xmlseckey );
	 
	 p_key = (php_xmlsec_key* ) key_result;
	 tpl_rsc ->p_key = p_key;
	 
	
	fprintf(  stderr, "Key  data is %d \n Key Addr is %x, KeyName %s , key_size %d , KeyAddr",   p_key->type , p_key , p_key->name,  strlen(p_key->name)  , &(p_key->name));
	
//	len= sprintf(string, "DOM  %s  \n len %d",  docStr , len);
	//RETURN_ZVAL(   );
}
/* }}} */
 
    /* {{{ proto string xmlsec_key_getdoc( rsrc template )
       this is test function */
PHP_FUNCTION( xmlsec_template_getdoc )
{
	int *arg = NULL;
	int len, k;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc, *tpl_rsc2;
	zval **result ,  *tpl_result;
	xmlDoc* pDoc ;
	char  *docStr;
	
	if ( ZEND_NUM_ARGS() != 1 ) WRONG_PARAM_COUNT;

	zend_get_parameters_ex(1, &result  );
	
	ZEND_FETCH_RESOURCE( tpl_result , php_xmlsec_tpl*, result , -1, "xmlsec template", le_xmlsectmpl  );
	 
	tpl_rsc  = (php_xmlsec_tpl* ) tpl_result ;
	pDoc = (xmlDoc* ) tpl_rsc ->doc;

		if ( !pDoc){
		 php_error(E_ERROR, "XmlDom:  is null" );		
		}
	//xmlDocDump( stderr, pDoc );
	 xmlDocDumpMemory (  pDoc ,  &docStr, &len );
	
	RETURN_STRINGL(  docStr, len, 1 );
}
/* }}} */

    /* {{{ proto string xmlsec_template_dsig_create( rsrc template, const TYPE )
       this is test function */
       /*          */
PHP_FUNCTION( xmlsec_template_dsig_create )
{
	int *arg = NULL;
	int len, k, tpl_type;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc, *tpl_rsc2, tpl;
	zval **result ,  *tpl_result, *rsc;
	xmlDoc* pDoc ;	
	xmlDoc*  pTmplDoc;
	xmlNode* encDataNode;
	xmlNode* keyInfoNode ;
	char  *docStr;
	xmlSecTransformId  TransformId = NULL;
	php_xmlsec_key  key_rsc;	
	php_xmlsec_key*  p_key; 
	
	
	if ( ZEND_NUM_ARGS() != 2 ) WRONG_PARAM_COUNT;
		
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zl",&rsc, &tpl_type  ) == FAILURE) {
		return;
	}	

	#define  XMLSEC_AES128Cbc    			0x0101
#define  XMLSEC_AES192Cbc    			0x0102 
#define  XMLSEC_AES256Cbc    			0x0103

#define  XMLSEC_DES	    				0x0200
#define  XMLSEC_3DESCbc	    			0x0201

	switch (tpl_type) 
	{	
		case XMLSEC_AES128Cbc: 
		TransformId = xmlSecTransformAes128CbcId;		
		break;
		case XMLSEC_AES256Cbc: 
		TransformId = xmlSecTransformAes256CbcId;		
		break;
		case XMLSEC_AES192Cbc: 
		TransformId = xmlSecTransformAes192CbcId;		
		break;
		case XMLSEC_3DESCbc: 
		TransformId = xmlSecTransformDes3CbcId;		
		break;
		default:
		TransformId = xmlSecTransformDes3CbcId;				
	}
	if (  TransformId == NULL )
		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Error: the crypto-algotithm are not determine\n "); 	
		
	zend_get_parameters_ex(1, &result  );
	
	ZEND_FETCH_RESOURCE( tpl_result , php_xmlsec_tpl*, result , -1, "xmlsec template", le_xmlsectmpl  );
	 
	tpl_rsc  = (php_xmlsec_tpl* ) tpl_result ;
	pDoc = (xmlDoc* ) tpl_rsc ->doc;
	
	
	if ( tpl_rsc  == NULL  )
		fprintf(  stderr, " tpl_rsc  NULL " );//
	
	if ( tpl_rsc ->doc != NULL  )
		pDoc = (xmlDoc* ) tpl_rsc ->doc;
//	fprintf(stderr, "point 1\n" );	
//	xmlDocDumpMemory (  pDoc ,  &docStr, &len );
//	fprintf(stderr, "point 2\n" );	
//	fprintf(  stderr, "DOM  %s  \n len %d",  docStr , len );	

/*     !!!!     **********************************************   */	
	if ( tpl_rsc ->p_key != NULL  )	
		fprintf(stderr, "key  Addr %x   " ,  tpl_rsc -> p_key );		

	p_key = (php_xmlsec_key*  ) tpl_rsc ->p_key;
	
	//if ( p_key != NULL )	 fprintf(stderr, "key  Addr %x   " ,   p_key);
	
		
	//fprintf(stderr, "keyName  %s \n" ,  key_rsc->name );
	
	
	pTmplDoc = xmlNewDoc(BAD_CAST "1.0");
		if ( !pDoc){
		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Error: the xml data for encryption not determine\n "); 	
		}
		
		xmlNode* n = xmlNewNode(NULL, BAD_CAST "envelope");
//	       xmlNodeSetContent(n, BAD_CAST "content");
	       
	        xmlNode* root = xmlDocGetRootElement(pDoc );
		xmlNodePtr  root2 =   xmlDocCopyNodeList ( pDoc, root );

			       	
      /* create encryption template to encrypt XML file and replace 
     * its content with encryption result 
     mlNodePtr  xmlSecTmplEncDataCreate         (xmlDocPtr doc,
                                             xmlSecTransformId encMethodId,
                                             const xmlChar *id,
                                             const xmlChar *type,
                                             const xmlChar *mimeType,
                                             const xmlChar *encoding);
     */
     
	    encDataNode = xmlSecTmplEncDataCreate(pDoc, TransformId ,
				NULL, xmlSecTypeEncElement, NULL, NULL);
    	if(encDataNode == NULL) {
		fprintf(stderr, "Error: failed to create encryption template \n");
    	}  		
	
	//xmlNode*  pRootNode  = xmlCopyNode ( xmlDocGetRootElement(pDoc),  1);
	
/* we want to put encrypted data in the <enc:CipherValue/> node 	*/
    if(xmlSecTmplEncDataEnsureCipherValue(encDataNode) == NULL) {
	fprintf(stderr, "Error: failed to add CipherValue node\n");
	}


    /* add <dsig:KeyInfo/> and <dsig:KeyName/> nodes to put key name in the signed document */
    keyInfoNode = xmlSecTmplEncDataEnsureKeyInfo(encDataNode, NULL);
    if(keyInfoNode == NULL) {
	fprintf(stderr, "Error: failed to add key info\n");	
    }

	if ( p_key->name != NULL  &&   xmlSecTmplKeyInfoAddKeyName(keyInfoNode, p_key->name ) == NULL) {
	fprintf(stderr, "Error: failed to add key name\n");	
    }	
		
	xmlAddChild( n, root2  );
	xmlAddChild(  n ,  encDataNode );	
       xmlDocSetRootElement( pDoc, n);

	xmlDocDump( stderr, 	pDoc );
	// xmlDocDumpMemory (  pDoc ,  &docStr, &len );
	
	//fprintf(  stderr, "DOM  %s  \n len %d",  docStr , len );
	
//	len= sprintf(string, "DOM  %s  \n len %d",  docStr , len);
	//RETURN_ZVAL(   );

}
/* }}} */
   
    /* {{{ proto string xmlsec_template_enc_create( rsrc template, TYPE  )
         create the template for encryption 
	   */
PHP_FUNCTION( xmlsec_template_enc_create )
{
	int *arg = NULL;
	int len, k, tpl_type;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc, *tpl_rsc2, tpl;
	zval **result ,  *tpl_result, *rsc;
	xmlDoc* pDoc ;	
	xmlDoc*  pTmplDoc;
	xmlNode* encDataNode;
	xmlNode* keyInfoNode ;
	char  *docStr;
	xmlSecTransformId  TransformId = NULL;
	php_xmlsec_key  key_rsc;	
	php_xmlsec_key*  p_key; 
	
	
	if ( ZEND_NUM_ARGS() != 2 ) WRONG_PARAM_COUNT;
		
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zl",&rsc, &tpl_type  ) == FAILURE) {
		return;
	}	

	#define  XMLSEC_AES128Cbc    			0x0101
#define  XMLSEC_AES192Cbc    			0x0102 
#define  XMLSEC_AES256Cbc    			0x0103

#define  XMLSEC_DES	    				0x0200
#define  XMLSEC_3DESCbc	    			0x0201

	switch (tpl_type) 
	{	
		case XMLSEC_AES128Cbc: 
		TransformId = xmlSecTransformAes128CbcId;		
		break;
		case XMLSEC_AES256Cbc: 
		TransformId = xmlSecTransformAes256CbcId;		
		break;
		case XMLSEC_AES192Cbc: 
		TransformId = xmlSecTransformAes192CbcId;		
		break;
		case XMLSEC_3DESCbc: 
		TransformId = xmlSecTransformDes3CbcId;		
		break;
		default:
		TransformId = xmlSecTransformDes3CbcId;				
	}
	if (  TransformId == NULL )
		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Error: the crypto-algotithm are not determine\n "); 	
		
	zend_get_parameters_ex(1, &result  );
	
	ZEND_FETCH_RESOURCE( tpl_result , php_xmlsec_tpl*, result , -1, "xmlsec template", le_xmlsectmpl  );
	 
	tpl_rsc  = (php_xmlsec_tpl* ) tpl_result ;
	pDoc = (xmlDoc* ) tpl_rsc ->doc;
	
	
	if ( tpl_rsc  == NULL  )
		fprintf(  stderr, " tpl_rsc  NULL " );//
	
	if ( tpl_rsc ->doc != NULL  )
		pDoc = (xmlDoc* ) tpl_rsc ->doc;
//	fprintf(stderr, "point 1\n" );	
//	xmlDocDumpMemory (  pDoc ,  &docStr, &len );
//	fprintf(stderr, "point 2\n" );	
//	fprintf(  stderr, "DOM  %s  \n len %d",  docStr , len );	

/*     !!!!     **********************************************   */	
	//if ( tpl_rsc ->p_key != NULL  )	fprintf(stderr, "key  Addr %x   " ,  tpl_rsc -> p_key );		

	p_key = (php_xmlsec_key*  ) tpl_rsc ->p_key;
	
	pTmplDoc = xmlNewDoc(BAD_CAST "1.0");
	
		if ( !pDoc){
		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Error: the xml data for encryption not determine\n "); 	
		}
		
      /* create encryption template to encrypt XML file and replace 
     * its content with encryption result 
     mlNodePtr  xmlSecTmplEncDataCreate         (xmlDocPtr doc,
                                             xmlSecTransformId encMethodId,
                                             const xmlChar *id,
                                             const xmlChar *type,
                                             const xmlChar *mimeType,
                                             const xmlChar *encoding);
     */
     
        encDataNode = xmlSecTmplEncDataCreate(pTmplDoc , TransformId ,
				NULL, xmlSecTypeEncElement, NULL, NULL);
    	if(encDataNode == NULL) {
		php_error(E_ERROR, "Error: failed to create encryption template \n ");		
    	}  		
	
//xmlNode*  pRootNode  = xmlCopyNode ( xmlDocGetRootElement(pDoc),  1);
	
/* we want to put encrypted data in the <enc:CipherValue/> node 	*/
    if(xmlSecTmplEncDataEnsureCipherValue(encDataNode) == NULL) {
	php_error(E_ERROR, "Error: failed to add CipherValue node\n ");			
	}


    /* add <dsig:KeyInfo/> and <dsig:KeyName/> nodes to put key name in the signed document */
    keyInfoNode = xmlSecTmplEncDataEnsureKeyInfo(encDataNode, NULL);
    if(keyInfoNode == NULL) {
    	php_error(E_ERROR, "Error: failed to add key \n ");				
    }

	if ( p_key->name != NULL  &&   xmlSecTmplKeyInfoAddKeyName(keyInfoNode, p_key->name ) == NULL) {
	php_error(E_ERROR, "Error: failed to add key name\n ");					
    }	
		
	xmlAddChild(  pTmplDoc  ,  encDataNode );	

	/*  save in the resource xmlDoc Template */

	tpl_rsc ->doc =  pTmplDoc;
	
	//xmlDocDump( stderr, 	pTmplDoc  );
	// xmlDocDumpMemory (  pDoc ,  &docStr, &len );
	//fprintf(  stderr, "DOM  %s  \n len %d",  docStr , len );
//	len= sprintf(string, "DOM  %s  \n len %d",  docStr , len);
	//RETURN_ZVAL(   );
}
/* }}} */




   /* {{{ proto string xmlsec_key_ini( int  type_key )
   */
PHP_FUNCTION( xmlsec_key_ini )
{
	int *arg = NULL;
	int len, k, keyType;
	int rsc_id;
	char string[256];
	php_xmlsec_key *key_rsc, *key_rsc2;
	zval **result ,  *key_result;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &arg ) == FAILURE) {
		return;
	}
	keyType = ( int)arg;
//	fprintf(  stderr, " test  of keey %d  \n",  keyType );
	
	if ( !( arg == XMLSEC_DES  ||  arg == XMLSEC_AES ))
			php_error(E_ERROR, "The key type is wrong ( type_id is %x) \nYou must to use XMLSEC key constant " , arg);
		
	key_rsc = (php_xmlsec_key* )  emalloc(sizeof( php_xmlsec_key) );

	key_rsc->type = (int) arg;
	
	//fprintf(  stderr, "is it test Addr  %x  \n",  key_rsc );
	 
	 ZEND_REGISTER_RESOURCE(return_value , key_rsc ,  le_xmlseckey );
	
}
/* }}} */

/* {{{ proto string xmlsec_key_setName(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_key_set_name )
{
	char *arg = NULL;
	int arg_len, len;
	char string[256];
	zval **result ,  *key_result;
	php_xmlsec_key  key_rsc;	
	php_xmlsec_key*  p_key,  pkey_rsc; 
	zval *res ;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &result  , &arg, &arg_len) == FAILURE) {
		php_error(E_ERROR, "The arguments is wrong \n" );
		return;
	}
	
	zend_get_parameters_ex(1, &result  );
	
	ZEND_FETCH_RESOURCE( res , php_xmlsec_key*, result , -1, "xmlsec results", le_xmlseckey );
	 p_key = (php_xmlsec_key* ) res;
	 
	 memset( p_key->name , 0,32 );
	 strncpy( p_key->name ,arg, arg_len );
	
	//len= sprintf(string, "addr %x  \nand nameAddr %x  \n",   p_key, *(p_key->name) );
//	fprintf(stderr , "name %s\n  addr %x  \nnameAddr %x , size %d \n", p_key->name,  p_key, &(p_key->name), arg_len);
	//RETURN_STRINGL(string, len, 1);

}
/* }}} */


/* {{{ proto string xmlsec_key_setName(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_key_get_name )
{
	zval **result ;
	int arg_len, len;
	char string[256];
	php_xmlsec_key  key_rsc;
	typedef  php_xmlsec_key* php_xmlsec_key_ptr;
	php_xmlsec_key_ptr  p_key,  pkey_rsc; 
	zval *res ;
	
          zend_get_parameters_ex(1, &result );
	   
	 ZEND_FETCH_RESOURCE( res , php_xmlsec_key*, result , -1, "xmlsec results", le_xmlseckey );
	 
	 p_key = (php_xmlsec_key_ptr  ) res;
	 len = strlen( p_key->name);
	fprintf(stderr , "name %s\n  addr %x  \nand nameAddr %x  len=%d \n", p_key->name,  p_key, &(p_key->name),  len );
	
	//fprintf(  stderr, "test 4  res  data  %d\n" , p_key->type );
	 
	len = sprintf(string, "\n  and Keyname is %s  ",    p_key->name );
	RETURN_STRINGL(string, len, 1);
}
/* }}} */

/* {{{ proto string xmlsec_key_setName(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_key_load )
{
	char *arg = NULL;
	zval **result ;
	int arg_len, len;
	char string[256];
	php_xmlsec_key  key_rsc;
	typedef  php_xmlsec_key* php_xmlsec_key_ptr;
	php_xmlsec_key_ptr  p_key,  pkey_rsc; 
	zval *res ;
	xmlSecKeyPtr pKey; 
	xmlSecSize keySize;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &result  , &arg, &arg_len) == FAILURE) {
		php_error(E_ERROR, "The arguments is wrong \n" );
		return;
	}
	
          zend_get_parameters_ex(1, &result );
	   
	 ZEND_FETCH_RESOURCE( res , php_xmlsec_key*, result , -1, "xmlsec results", le_xmlseckey );

	 p_key = (php_xmlsec_key_ptr  ) res;
	 bzero( p_key->key ,2048 );
	 strncpy( p_key->key ,arg, arg_len );
	 
	int keyType =  (int) p_key->type;
	
	keySize =  strlen( arg ) ;
	printf(  stderr, "the  key sioze  %d  \n \n",  keySize  );	
	
	
	
	switch( keyType ){
		case XMLSEC_DES:  
			pKey = xmlSecKeyReadMemory (xmlSecKeyDataAesId  , (xmlSecByte* ) arg   ,  keySize );
		case XMLSEC_AES: 
			pKey = xmlSecKeyReadMemory (xmlSecKeyDataDesId  , (xmlSecByte* ) arg   ,  keySize );
/*	
Reads the key value of klass dataId from a memory block data.
Returns :pointer to newly created key or NULL if an error occurs.
xmlSecKeyPtr xmlSecKeyReadMemory (xmlSecKeyDataId dataId, - the key value data klass.
 								const xmlSecByte *data, 	the memory containing the key
								xmlSecSize dataSize);		the size of the memory block
*/
		break;
		default: 
			php_error(E_ERROR, "The key type is wrong.");
	}

	if (pKey == NULL  )
		php_error(E_WARNING, "the xmlSecKeyReadMemory  aborted\n" );
	else
	   	p_key->p_key = pKey;

		 	 
	
//	p_key->key  
//	fprintf(  stderr, "test  data  of pKey %x\n" , p_key->p_key );
	 
	//len = sprintf(string, "\n key is %s  len is %d\n ",   p_key->key  , strlen( p_key->key) );
	//RETURN_STRINGL(string, len, 1);
}
/* }}} */


/* {{{ proto string  xmlsec_doc_ini(string arg)
   initialize xmlsec_doc resource and save xmlDoc from xml string   
   */
PHP_FUNCTION( xmlsec_doc_ini )
{
	int *arg_len = NULL;
	char *arg_str = NULL;
	zval *arg_rsc; 
	
	int len, k, keyType;
	int rsc_id;
	char string[256];
	php_xmlsec_doc *doc_rsc;
	zval **result ,  *key_result;
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &arg_str, &arg_len ) == FAILURE) {
		php_error(E_ERROR, "The  wrong parameters " );
		return;
	}

	doc_rsc = (php_xmlsec_doc* )     emalloc(sizeof( php_xmlsec_doc ));
	if ( ZEND_NUM_ARGS() == 1 )
	{		
		xmlDoc*pDoc =   xmlParseMemory( arg_str, arg_len);
		if ( !pDoc){
		 php_error(E_ERROR, "XmlDom:  c'not parse xml document" );		
		}
		//fprintf(  stderr, "the doc addr %x \n",  pDoc);
		doc_rsc->doc =  pDoc;
		
		//xmlDocDump( stderr, 	pDoc  );
	}
//	fprintf(  stderr, "is it template rsc Addr  %x  \n",  tpl_rsc );
	
	  ZEND_REGISTER_RESOURCE(return_value , doc_rsc ,  le_xmlsecdoc  );
}
/* }}} */


/* {{{ proto string  xmlsec_doc_test(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_doc_set_template )
{
	int *arg = NULL;
	int len, k;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc;
//	php_xmlsec_key*  p_key;
	php_xmlsec_doc  *doc_rsc, *p_doc;
	
	zval **result ,  *doc_result , **result_2 , *tpl_result;
	xmlDoc* pDoc ;
	char  *docStr;
	
	
	if ( ZEND_NUM_ARGS() != 2 ) WRONG_PARAM_COUNT;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rr", &result  , &result_2  ) == FAILURE) {
		php_error(E_ERROR, "The arguments is wrong \n" );
		return;
	}	 

	zend_get_parameters_ex(2, &result  , &result_2);	
	
	ZEND_FETCH_RESOURCE( doc_result,  php_xmlsec_doc*, result , -1, "xmlsec doc ", le_xmlsecdoc );
	 
	 p_doc = (php_xmlsec_doc* ) doc_result;


	ZEND_FETCH_RESOURCE( tpl_result , php_xmlsec_tpl*, result_2 , -1, "xmlsec template", le_xmlsectmpl  );	 
	tpl_rsc  = (php_xmlsec_tpl* ) tpl_result ;
//	pDoc = (xmlDoc* ) tpl_rsc ->doc;
//	xmlDocDump( stderr, 	pDoc );		
	p_doc->p_tpl = tpl_rsc ;

}
/* }}} */


/* {{{ proto string  xmlsec_doc_test(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION( xmlsec_doc_encrypt )
{
	int *arg = NULL;
	int len, k;
	int *pLen;
	int rsc_id;
	char string[256];
	php_xmlsec_tpl *tpl_rsc, *p_tpl = NULL;
	php_xmlsec_key *p_key, *p_key_2;
	php_xmlsec_doc  *doc_rsc, *p_doc;
		
	zval **result ,  *doc_result , **result_2 , *tpl_result;
	xmlDoc *pDocXml , *pDocTpl ;
	char  *docStr;
	
	xmlSecEncCtxPtr encCtx;	
	xmlNodePtr encDataNode; 
	
	zend_get_parameters_ex(1, &result  );	
	
	ZEND_FETCH_RESOURCE( doc_result,  php_xmlsec_doc*, result , -1, "xmlsec doc ", le_xmlsecdoc );
	 p_doc = (php_xmlsec_doc* ) doc_result; // xml data doc for encryption
	 
	 if ( p_doc->doc == NULL ) 
	 	php_error(E_ERROR, "The incoming data xml document is null or don't valid. \n" );
	
	p_tpl = (php_xmlsec_tpl*) p_doc->p_tpl ; 

	if ( p_tpl == NULL) 
		php_error(E_ERROR, "The template is null or don't valid. \n" );	
	 
	p_key = (php_xmlsec_key*)p_doc->p_key;	
	p_key_2 = (php_xmlsec_key*)p_tpl->p_key;
	
	 xmlSecKeyPtr  p_key_in_php_key = ( xmlSecKeyPtr ) p_key_2->p_key;
	
	if ( p_key_in_php_key  == NULL)
	 		php_error(E_ERROR, "The key in the xmlsec_key is null. \n" );
	
	/*         
	   	  p_key   p_key_2 
	  p_key 	*/
	if (    p_key_2 == NULL)
		php_error(E_ERROR, "The xmlsec key 2 resource is null. \n" );
	
	/*  create encryption context */
	encCtx =xmlSecEncCtxCreate(NULL) ;	
	
	encCtx->encKey =  p_key_2->p_key;
	    
	    /* encrypt the data */
    	pDocTpl = (xmlDoc* ) p_tpl->doc;   // template
	
   /* find start node */
    encDataNode = xmlSecFindNode(xmlDocGetRootElement( pDocTpl ), xmlSecNodeEncryptedData, xmlSecEncNs);
    xmlNode* root = xmlDocGetRootElement(p_doc->doc);
    
    if( encDataNode  == NULL) {
	php_error(E_ERROR,  "Error: start node not found in template\n"  );		
    }

    /*   	
    	Encrypts node according to template tmpl. If requested, node is replaced with result <enc:EncryptedData/> node.
	int  xmlSecEncCtxXmlEncrypt (xmlSecEncCtxPtr encCtx, 	the pointer to <enc:EncryptedData/> processing context.
                                             xmlNodePtr tmpl,				the pointer to <enc:EncryptedData/> template node.
                                             xmlNodePtr node);				the pointer to node for encryption.
	return  0 on success or a negative value if an error occurs.					     
*/
	if(xmlSecEncCtxXmlEncrypt(encCtx, encDataNode,  root ) < 0) {
        	php_error(E_ERROR,  "Error: encryption failed\n"  );				
    }

	//xmlDocDump( stderr,  p_doc->doc);		
	xmlDocDumpMemory (  p_doc->doc,  &docStr, &len );

	RETURN_STRINGL(  docStr, len,  1 ) ;
	encDataNode  = NULL;

	/*  destroy encCtx  */
        if(encCtx != NULL) {
		xmlSecEncCtxDestroy(encCtx);
    	}

	

}
		
/* }}} */

